@@ -23,13 +23,13 @@ import path_to_kapture
 # import kapture
 import kapture.utils.logging
 
-logger = logging.getLogger('download')
+logger = logging.getLogger('downloader')
 logging.basicConfig(format='%(levelname)-8s::%(name)s: %(message)s')
 
 INDEX_FILENAME = 'kapture_dataset_index.yaml'
 DEFAULT_DATASET_PATH = path.normpath(path.abspath('.'))
-# DEFAULT_REPOSITORY_URL = 'https://github.com/naver/kapture/raw/master/dataset'
-DEFAULT_REPOSITORY_URL = 'https://download.europe.naverlabs.com/kapture/'
+DEFAULT_REPOSITORY_URL = 'https://github.com/naver/kapture/raw/master/dataset'
+# DEFAULT_REPOSITORY_URL = 'https://download.europe.naverlabs.com/kapture/'
 datasets = {}
 
 
@@ -276,12 +276,9 @@ class Dataset:
             install_script_filepath = path.join(self._install_local_path, self._install_script_filename)
             logger.info(f'applying installation script {install_script_filepath}')
             ret = os.system(install_script_filepath)
-            # print(ret);
-            # exit()
-            # os.remove(install_script_filepath)
 
         # done
-        # self.set_status('installed')
+        self.set_status('installed')
         logger.info(f'done installing {self._name}')
 
 
